// vector4 class
// -Kyoung Shin Park (2011 Spring)
//////////////////////////////////////////////////////////////////////
#include "vector4.h"

namespace oglclass
{

vector4 vector4::Zero(0.0, 0.0, 0.0, 0.0);

vector4::vector4(const vector4 &other)
{
    v_[0] = other.v_[0];
    v_[1] = other.v_[1];
    v_[2] = other.v_[2];
    v_[3] = other.v_[3];
}

vector4::vector4(const float *other)
{
    v_[0] = other[0];
    v_[1] = other[1];
    v_[2] = other[2];
    v_[3] = other[3];
}

vector4::vector4(float x, float y, float z, float w)
{
    v_[0] = x;
    v_[1] = y;
    v_[2] = z;
    v_[3] = w;
}

void vector4::set(float x, float y, float z, float w)
{
    v_[0] = x;
    v_[1] = y;
    v_[2] = z;
    v_[3] = w;
}

float vector4::length(void)
{
    return ((float)sqrt( v_[0] * v_[0] + v_[1] * v_[1] + v_[2] * v_[2] + v_[3] * v_[3]));
}

void vector4::normalize(void)
{
    float fLength = length();

    v_[0] = v_[0] / fLength;
    v_[1] = v_[1] / fLength;
    v_[2] = v_[2] / fLength;
    v_[3] = v_[3] / fLength;
}

float vector4::distance(const vector4 &v1,  const vector4 &v2)
{
    float dx = v1.v_[0] - v2.v_[0];
    float dy = v1.v_[1] - v2.v_[1];
    float dz = v1.v_[2] - v2.v_[2];
    float dw = v1.v_[3] - v2.v_[3];

    return (float)sqrt(dx * dx + dy * dy + dz * dz + dw * dw);
}

float vector4::dotProduct(const vector4 &v1,  const vector4 &v2)
{
    return (v1.v_[0] * v2.v_[0] + v1.v_[1] * v2.v_[1] + v1.v_[2] * v2.v_[2] + v1.v_[3] * v2.v_[3]);
}

// operators...
vector4& vector4::operator = (const vector4 &other)
{
    v_[0] = other.v_[0];
    v_[1] = other.v_[1];
    v_[2] = other.v_[2];
    v_[3] = other.v_[3];

    return *this;
}

vector4& vector4::operator += (const vector4 &other)
{
    v_[0] += other.v_[0];
    v_[1] += other.v_[1];
    v_[2] += other.v_[2];
    v_[3] += other.v_[3];

    return *this;
}

vector4& vector4::operator -= (const vector4 &other)
{
    v_[0] -= other.v_[0];
    v_[1] -= other.v_[1];
    v_[2] -= other.v_[2];
    v_[3] -= other.v_[3];

    return *this;
}

vector4& vector4::operator *= (const float scalar)
{
    v_[0] *= scalar;
    v_[1] *= scalar;
    v_[2] *= scalar;
    v_[3] *= scalar;

    return *this;
}

vector4& vector4::operator /= (const float scalar)
{
    v_[0] /= scalar;
    v_[1] /= scalar;
    v_[2] /= scalar;
    v_[3] /= scalar;

    return *this;
}

bool vector4::operator == (const vector4 &other) const
{
	return (v_[0] == other.v_[0] && v_[1] == other.v_[1] && v_[2] == other.v_[2] && v_[3] == other.v_[3]);
}

bool vector4::operator != (const vector4 &other) const
{
	return (v_[0] != other.v_[0] || v_[1] != other.v_[1] || v_[2] != other.v_[2] || v_[3] != other.v_[3]);
}

vector4 vector4::operator + (const vector4 &other)
{
    vector4 result(0.0f, 0.0f, 0.0f, 0.0f);

    result.v_[0] = v_[0] + other.v_[0];
    result.v_[1] = v_[1] + other.v_[1];
    result.v_[2] = v_[2] + other.v_[2];
    result.v_[3] = v_[3] + other.v_[3];

    return result;
}

vector4 vector4::operator - (const vector4 &other)
{
    vector4 result(0.0f, 0.0f, 0.0f, 0.0f);

    result.v_[0] = v_[0] - other.v_[0];
    result.v_[1] = v_[1] - other.v_[1];
    result.v_[2] = v_[2] - other.v_[2];
    result.v_[3] = v_[3] - other.v_[3];

    return result;
}

vector4 vector4::operator * (const vector4 &other)
{
    vector4 result(0.0f, 0.0f, 0.0f, 0.0f);

    result.v_[0] = v_[0] * other.v_[0];
    result.v_[1] = v_[1] * other.v_[1];
    result.v_[2] = v_[2] * other.v_[2];
    result.v_[3] = v_[3] * other.v_[3];

    return result;
}

vector4 vector4::operator / (const vector4 &other)
{
    vector4 result(0.0f, 0.0f, 0.0f, 0.0f);

    result.v_[0] = v_[0] / other.v_[0];
    result.v_[1] = v_[1] / other.v_[1];
    result.v_[2] = v_[2] / other.v_[2];
    result.v_[3] = v_[3] / other.v_[3];

    return result;
}

vector4 vector4::operator * (const float scalar)
{
    vector4 result(0.0f, 0.0f, 0.0f, 0.0f);

    result.v_[0] = v_[0] * scalar;
    result.v_[1] = v_[1] * scalar;
    result.v_[2] = v_[2] * scalar;
    result.v_[3] = v_[3] * scalar;

    return result;
}

vector4 vector4::operator / (const float scalar)
{
    vector4 result(0.0f, 0.0f, 0.0f, 0.0f);

    result.v_[0] = v_[0] / scalar;
    result.v_[1] = v_[1] / scalar;
    result.v_[2] = v_[2] / scalar;
    result.v_[3] = v_[3] / scalar;

    return result;
}

vector4 operator * (const float scalar, const vector4 &other)
{
    vector4 result(0.0f, 0.0f, 0.0f, 0.0f);

    result.v_[0] = other.v_[0] * scalar;
    result.v_[1] = other.v_[1] * scalar;
    result.v_[2] = other.v_[2] * scalar;
    result.v_[3] = other.v_[3] * scalar;

    return result;
}

vector4 operator / (const float scalar, const vector4 &other)
{
    vector4 result(0.0f, 0.0f, 0.0f, 0.0f);

    result.v_[0] = other.v_[0] / scalar;
    result.v_[1] = other.v_[1] / scalar;
    result.v_[2] = other.v_[2] / scalar;
    result.v_[3] = other.v_[3] / scalar;

    return result;
}

vector4 vector4::operator + (void) const
{
    return *this;
}

vector4 vector4::operator - (void) const
{
    vector4 result(-v_[0], -v_[1], -v_[2], -v_[3]);

    return result;
}

float vector4::operator [] (const int i) const
{
	if (i == 0)			// if i==0, return x
		return v_[0];
	else if (i == 1)	// else if i==1, return y
		return v_[1];
	else if (i == 2)	// else if i==2, return z
		return v_[2];
	return v_[3];			// otherwise, return w
}

float& vector4::operator [] (const int i)
{
	if (i == 0)			// if i==0, return x
		return v_[0];
	else if (i == 1)	// else if i==1, return y
		return v_[1];
	else if (i == 2)	// else if i==2, return z
		return v_[2];
	return v_[3];			// otherwise, return w
}

} // namespace