// vector4 class
// -Kyoung Shin Park (2011 Spring)
//////////////////////////////////////////////////////////////////////
#ifndef _VECTOR4_H_
#define _VECTOR4_H_

#include <cmath>
#include <iostream>
using namespace std;

namespace oglclass
{

class vector4
{
public:
    static vector4 Zero;

	// constructor
	vector4(void) { v_[0] = v_[1] = v_[2] = v_[3] = 0.0f; }
	vector4(const vector4 &other);
    vector4(const float *other);
    vector4(float x, float y, float z, float w);
    void set(float x, float y, float z, float w);

	// length & normalize
    float length(void);
    void normalize(void);

	// static utility methods
    static float distance(const vector4 &v1, const vector4 &v2);
    static float dotProduct(const vector4 &v1, const vector4 &v2 );

    // assignment operators...
    vector4& operator = (const vector4 &other);
    vector4& operator += (const vector4 &other);
    vector4& operator -= (const vector4 &other);
    vector4& operator *= (const float scalar);
    vector4& operator /= (const float scalar);

	// equality operators
	bool operator == (const vector4 &other) const;
	bool operator != (const vector4 &other) const;

	// vector operators
    vector4 operator + (const vector4 &other);
    vector4 operator - (const vector4 &other);
    vector4 operator * (const vector4 &other);
    vector4 operator / (const vector4 &other);

    vector4 operator * (const float scalar);
    friend vector4 operator * (const float scalar, const vector4 &other);
    vector4 operator / (const float scalar);
    friend vector4 operator / (const float scalar, const vector4 &other);
   
    vector4 operator + (void) const;
    vector4 operator - (void) const;

	float operator[] (const int i) const;
	float& operator[] (const int i);
	operator float* () {	return v_; } // type conversion operator overloading

	friend ostream& operator<< (ostream& s, const vector4& v);

protected:
	float v_[4];
};

inline ostream& operator<< (ostream& s, const vector4& v)
{ 
	return s << "VECTOR4 (" << v[0] << ", " << v[1] << ", " << v[2] << ", " << v[3] << ")"; 
}

} // namespace

#endif
